package at.fhooe.mc.magicwand.client.test;

import at.fhooe.mc.magicwand.client.exceptions.CompassException;
import at.fhooe.mc.magicwand.client.exceptions.GPSException;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import at.fhooe.mc.magicwand.client.sensors.*;

/**
 * @class SensorTestMIDlet
 * @brief this MIDlet can be used to test the socket to the server. a socket connection
 * is established (port 4444) when the MIDlet is started. By clicking on "Send/Receive", you
 * can test if the data transfer is working. The connection can be closed by clicking
 * on "Exit".
 * @author Manuela Vogler
 */
public class SensorTestMIDlet extends MIDlet implements CommandListener {

    private boolean midletPaused = false;
    private Command exitCommand;
    private Command itemCommand;
    private Form form;
    private StringItem stringItem;
    private StringItem stringItemLatitude;
    private StringItem stringItemLongitude;
    private StringItem stringItemAngle;
    private SensorDataContainer m_dataContainer;

    /**
     * @fn public SensorTestMIDlet()
     * @brief The HelloMIDlet constructor.
     */
    public SensorTestMIDlet() {
    }

    /**
     * @fn private void initialize()
     * @brief Initilizes the application.
     * It is called only once when the MIDlet is started. The method is called before the startMIDlet method.
     */
    private void initialize() {
        try {
            m_dataContainer = new SensorDataContainer();
        } catch (CompassException _ex) {
            _ex.printStackTrace();
        }
    }

    /**
     * @fn public void startMIDlet()
     * @brief Performs an action assigned to the Mobile Device - MIDlet Started point.
     */
    public void startMIDlet() {
        switchDisplayable(null, getForm());

    }

    /**
     * @fn public void resumeMIDlet()
     * @brief Performs an action assigned to the Mobile Device - MIDlet Resumed point.
     */
    public void resumeMIDlet() {
    }

    /**
     * @fn public void switchDisplayable(Alert _alert, Displayable _nextDisplayable)
     * @brief Switches a current displayable in a display. The <code>display</code> instance is taken from <code>getDisplay</code> method. This method is used by all actions in the design for switching displayable.
     * @param _alert the Alert which is temporarily set to the display; if <code>null</code>, then <code>nextDisplayable</code> is set immediately
     * @param _nextDisplayable the Displayable to be set
     */
    public void switchDisplayable(Alert _alert, Displayable _nextDisplayable) {
        Display display = getDisplay();
        if (_alert == null) {
            display.setCurrent(_nextDisplayable);
        } else {
            display.setCurrent(_alert, _nextDisplayable);
        }
    }

    /**
     * @fn public void commandAction(Command _command, Displayable _displayable)
     * @brief Called by a system to indicated that a command has been invoked on a particular displayable.
     * @param _command the Command that was invoked
     * @param _displayable the Displayable where the command was invoked
     */
    public void commandAction(Command _command, Displayable _displayable) {
        if (_displayable == form) {
            try {
                if (_command == exitCommand) {
                    m_dataContainer.close();
                    exitMIDlet();
                } else if (_command == itemCommand) {
                    m_dataContainer.updateData();
                    stringItemLatitude.setText(String.valueOf(m_dataContainer.getLatitude()));
                    stringItemLongitude.setText(String.valueOf(m_dataContainer.getLongitude()));
                    stringItemAngle.setText(m_dataContainer.getAngle());
                }
            } catch (CompassException _ex) {
                _ex.printStackTrace();
            } catch (GPSException _ex) {
                _ex.printStackTrace();
            }
        }

    }

    /**
     * @fn public Command getExitCommand()
     * @brief Returns an initiliazed instance of exitCommand component.
     * @return the initialized component instance
     */
    public Command getExitCommand() {
        if (exitCommand == null) {
            exitCommand = new Command("Exit", Command.EXIT, 0);
        }
        return exitCommand;
    }

    /**
     * @fn public Form getForm()
     * @brief Returns an initiliazed instance of form component.
     * @return the initialized component instance
     */
    public Form getForm() {
        if (form == null) {
            form = new Form("Welcome", new Item[]{getStringItem(), getStringItemLatitude(), getStringItemLongitude(), getStringItemAngle()});
            form.addCommand(getExitCommand());
            form.addCommand(getItemCommand());
            form.setCommandListener(this);

        }
        return form;
    }

    /**
     * @fn public StringItem getStringItem()
     * @brief Returns an initiliazed instance of stringItem component.
     * @return the initialized component instance
     */
    public StringItem getStringItem() {
        if (stringItem == null) {

            stringItem = new StringItem("TestMidlet", "Testing Sensor Requests");
        }
        return stringItem;
    }

    /**
     * @fn public Command getItemCommand()
     * @brief Returns an initiliazed instance of itemCommand component.
     * @return the initialized component instance
     */
    public Command getItemCommand() {
        if (itemCommand == null) {

            itemCommand = new Command("Send/Receive", Command.ITEM, 0);

        }
        return itemCommand;
    }

    /**
     * @fn public StringItem getStringItemLatitude()
     * @brief Returns an initiliazed instance of stringItemLatitude component.
     * @return the initialized component instance
     */
    public StringItem getStringItemLatitude() {
        if (stringItemLatitude == null) {

            stringItemLatitude = new StringItem("Latitude", "");

        }
        return stringItemLatitude;
    }

    /**
     * @fn public StringItem getStringItemLongitude()
     * @brief Returns an initiliazed instance of stringItemLongitude component.
     * @return the initialized component instance
     */
    public StringItem getStringItemLongitude() {
        if (stringItemLongitude == null) {

            stringItemLongitude = new StringItem("Longitude", "");

        }
        return stringItemLongitude;
    }

    /**
     * @fn public StringItem getStringItemAngle()
     * @brief Returns an initiliazed instance of stringItemAngle component.
     * @return the initialized component instance
     */
    public StringItem getStringItemAngle() {
        if (stringItemAngle == null) {
            stringItemAngle = new StringItem("Angle", "");
        }
        return stringItemAngle;
    }

    /**
     * @fn public Display getDisplay()
     * @brief Returns a display instance.
     * @return the display instance.
     */
    public Display getDisplay() {
        return Display.getDisplay(this);
    }

    /**
     * @fn public void exitMIDlet()
     * @brief Exits MIDlet.
     */
    public void exitMIDlet() {
        switchDisplayable(null, null);
        destroyApp(true);
        notifyDestroyed();
    }

    /**
     * @fn public void startApp()
     * @brief Called when MIDlet is started.
     * Checks whether the MIDlet have been already started and initialize/starts or resumes the MIDlet.
     */
    public void startApp() {
        if (midletPaused) {
            resumeMIDlet();
        } else {
            initialize();
            startMIDlet();
        }
        midletPaused = false;
    }

    /**
     * @fn public void pauseApp()
     * @brief Called when MIDlet is paused.
     */
    public void pauseApp() {
        midletPaused = true;
    }

    /**
     * @fn public void destroyApp(boolean unconditional)
     * @brief Called to signal the MIDlet to terminate.
     * @param unconditional if true, then the MIDlet has to be unconditionally terminated and all resources has to be released.
     */
    public void destroyApp(boolean unconditional) {
    }
}
